#include "../../../include/GPU/GPU.h"
#include "SimpleHelper.hpp"
#include <stdio.h>

namespace FileProcessor {
    using GPU::PositionU16;
    using GPU::SizeU16;

    struct SimpleTIMSize : private SimpleTIM {
        constexpr SimpleTIMSize() {
        }

        constexpr uint16_t getTextureWidth() const {
            return SimpleTIM::getTextureX();
        }

        constexpr uint16_t getTextureHeight() const {
            return SimpleTIM::getTextureY();
        }

        constexpr uint16_t getClutWidth() const {
            return SimpleTIM::getClutX();
        }

        constexpr uint16_t getClutHeight() const {
            return SimpleTIM::getClutY();
        }
    };

    struct SimpleTIMState {
        SimpleTIM     dst_info;
        SimpleTIMSize size_info;        

        constexpr SimpleTIMState(const SimpleTIM& dst_info) : dst_info(dst_info) {
        }
    };

    static void set_gpu_receive(const uint32_t* src, uint16_t x, uint16_t y, uint16_t w, uint16_t h) {
        GPU::DMA::Receive::prepare();
        GPU::DMA::Receive::set_dst(PositionU16(x, y), SizeU16(w, h)); 
        GPU::DMA::Receive::set_src(reinterpret_cast<const uintptr_t>(src));
    }

    static void set_gpu_receive_data(const uint32_t* src, const SimpleTIMState& state) {
        set_gpu_receive(src, state.dst_info.getTextureX(), state.dst_info.getTextureY(), state.size_info.getTextureWidth(), state.size_info.getTextureWidth());
    }

    static bool parse_data(State::Configuration& config, SimpleTIMState& state) {
        return false;
    }

    static bool parse_header(State::Configuration& config, SimpleTIMState& state) {
        if(config.data_size >= sizeof(SimpleTIMSize)) {
            Helper::simple_read(state.size_info, config);
            
            //Check if we have a clut to care about
            if(state.size_info.getClutWidth() > 0) {
                //CLUTs are 16bit full color anyway
                printf("We found a CLUT which is not yet implemented!\n");
                return false;
            }

            //We have direct data
            else {
                set_gpu_receive_data(config.data_adr, state);
                return Helper::exchange_and_execute_process_function(parse_data, config, state);
            }
        }

        return true;
    }

    State create(const uint32_t* data_adr, const SimpleTIM& file) {        
        return State::from(SimpleTIMState(file), data_adr, parse_header);
    }
}

#undef private